import {
  ProtocolRequest,
  ProtocolResponse,
  ProtocolEncoder,
  PacketBody,
  PacketBodyValue,
  ProtocolDecoder,
  ProtocolMessage,
  BodyStringDivider
} from '@ucap-webmessenger/protocol';
import { UserInfo } from '../types/userInfo';

export interface BuddyInfo {
  // 사용자SEQ
  userSeq: number;
  // 즐.찾 여부
  isFavorit: boolean;
  // 친구여부
  isBuddy: boolean;
}

export interface SyncBuddyRequest extends ProtocolRequest {
  // 0. 동료 씽크일시(s) cf)2019-09-24 16:51:42
  syncDate: string;
}

export interface SyncBuddyDataResponse extends ProtocolResponse {
  // n. {동료정보-S}...
  buddyInfos: BuddyInfo[];
}

export interface SyncBuddyDataDetailResponse extends ProtocolResponse {
  // n. {동료정보-S}...
  buddyInfos: UserInfo[];
}

export interface SyncBuddyResponse extends ProtocolResponse {
  // 0. 동료 씽크일시(s) cf)2019-09-24 16:51:42
  syncDate: string;
}

export const encodeSyncBuddy: ProtocolEncoder<SyncBuddyRequest> = (
  req: SyncBuddyRequest
) => {
  const bodyList: PacketBody[] = [];

  bodyList.push({ type: PacketBodyValue.String, value: req.syncDate || '' });
  return bodyList;
};

export const decodeSyncBuddy: ProtocolDecoder<SyncBuddyResponse> = (
  message: ProtocolMessage
) => {
  return {
    syncDate: message.bodyList[0]
  } as SyncBuddyResponse;
};

export const decodeSyncBuddyData: ProtocolDecoder<SyncBuddyDataResponse> = (
  message: ProtocolMessage
) => {
  const buddyInfos: BuddyInfo[] = [];
  message.bodyList.forEach(buddyinfo => {
    const info = buddyinfo.split(BodyStringDivider);
    buddyInfos.push({
      userSeq: info[0],
      isFavorit: info[1] === 'Y' ? true : false,
      isBuddy: info[2] === 'Y' ? true : false
    });
  });
  return {
    buddyInfos
  } as SyncBuddyDataResponse;
};

export const decodeSyncBuddyDataDetail: ProtocolDecoder<
  SyncBuddyDataDetailResponse
> = (message: ProtocolMessage) => {
  const buddyInfos: UserInfo[] = [];
  message.bodyList.forEach(buddyinfo => {
    const info = buddyinfo.split(BodyStringDivider);
    let i = 0;
    buddyInfos.push({
      userSeq: info[i],
      name: info[i++],
      profileImageFile: info[i++],
      grade: info[i++],
      intro: info[i++],
      companyCode: info[i++],
      hpNumber: info[i++],
      lineNumber: info[i++],
      email: info[i++],
      isMobile: info[i++],
      deptName: info[i++],
      isFavorit: info[i++],
      isBuddy: info[i++],
      isActive: info[i++],
      roleCd: info[i++],
      employeeNum: info[i++],
      madn: info[i++],
      hardSadn: info[i++],
      fmcSadn: info[i++],
      nameEn: info[i++],
      nameCn: info[i++],
      gradeEn: info[i++],
      gradeCn: info[i++],
      deptNameEn: info[i++],
      deptNameCn: info[i++],
      isPrivacyAgree: info[i++],
      isValidLogin: info[i++],
      employeeType: info[i++],
      nickName: info[i++]
    });
  });
  return {
    buddyInfos
  } as SyncBuddyDataDetailResponse;
};
