import {
  ProtocolRequest,
  ProtocolResponse,
  ProtocolEncoder,
  PacketBody,
  PacketBodyValue,
  ProtocolDecoder,
  ProtocolMessage,
  BodyStringDivider
} from '@ucap-webmessenger/protocol';

export interface GroupInfo {
  // 그룹SEQ(n)
  seq: number;
  // 그룹이름(s)
  name: string;
  // ActiveYN(s)
  isActive: boolean;
  // 사용자SEQ(s)
  userSeqStr: string;
}
export interface SyncGroupRequest extends ProtocolRequest {
  // 0. 동료 씽크일시(s) cf)2019-09-24 16:51:42
  syncDate: string;
}

export interface SyncGroupDataResponse extends ProtocolResponse {
  // n. {동료그룹정보}...
  groupInfos: GroupInfo[];
}

export interface SyncGroupDataDetailResponse extends ProtocolResponse {
  // 그룹SEQ(n)
  seq: number;
  // 그룹이름(s)
  name: string;
  // ActiveYN(s)
  isActive: boolean;
  // 사용자SEQ(n)1
  userSeqs: number[];
}

export interface SyncGroupResponse extends ProtocolResponse {
  // 0. 동료 씽크일시(s) cf)2019-09-24 16:51:42
  syncDate: string;
}

export const encodeSyncGroup: ProtocolEncoder<SyncGroupRequest> = (
  req: SyncGroupRequest
) => {
  const bodyList: PacketBody[] = [];

  bodyList.push({ type: PacketBodyValue.String, value: req.syncDate || '' });
  return bodyList;
};

export const decodeSyncGroup: ProtocolDecoder<SyncGroupResponse> = (
  message: ProtocolMessage
) => {
  return {
    syncDate: message.bodyList[0]
  } as SyncGroupResponse;
};

export const decodeSyncGroupData: ProtocolDecoder<SyncGroupDataResponse> = (
  message: ProtocolMessage
) => {
  const groupInfos: GroupInfo[] = [];
  message.bodyList.forEach(buddyinfo => {
    const info = buddyinfo.split(BodyStringDivider);
    groupInfos.push({
      seq: info[0],
      name: info[1],
      isActive: info[2] === 'Y' ? true : false,
      userSeqStr: info[3]
    });
  });

  return {
    groupInfos
  } as SyncGroupDataResponse;
};

export const decodeSyncGroupDataDetail: ProtocolDecoder<
  SyncGroupDataDetailResponse
> = (message: ProtocolMessage) => {
  let userSeqs: number[] = [];
  if (message.bodyList.length > 3) {
    userSeqs = message.bodyList.slice(3);
  }
  return {
    // 그룹SEQ(n)
    seq: message.bodyList[0],
    // 그룹이름(s)
    name: message.bodyList[1],
    // ActiveYN(s)
    isActive: message.bodyList[2] === 'Y' ? true : false,
    // 사용자SEQ(n)1
    userSeqs
  } as SyncGroupDataDetailResponse;
};
